﻿using Capnp.Rpc;
using FabAccessAPI.Exceptions;
using FabAccessAPI.Schema;
using System;
using System.Collections.Generic;
using System.Net.Security;
using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Threading.Tasks;

namespace FabAccessAPI
{
    public class API : IAPI
    {
        #region Private Members
        private Connection _APIConnection;
        #endregion

        #region Constructors
        public API()
        {

        }
        #endregion

        #region Events
        public event EventHandler<ConnectionStatusChange> ConnectionStatusChanged;
        #endregion

        #region Members
        public ConnectionData ConnectionData { get; private set; }

        public ConnectionInfo ConnectionInfo { get; private set; }

        public bool IsConnected
        {
            get
            {
                return _APIConnection != null && _APIConnection.RpcClient.State == ConnectionState.Active;
            }
        }

        public Session Session
        {
            get
            {
                throw new NotImplementedException();
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Connect to server with ConnectionData
        /// </summary>
        /// <exception cref="AuthenticationException"></exception>
        /// <exception cref="ConnectingFailedException"></exception>
        public async Task Connect(ConnectionData connectionData)
        {
            if (IsConnected)
            {
                await Disconnect();
            }

            TcpRpcClient rpcClient = await _ConnectAsync(connectionData).ConfigureAwait(false);

            _APIConnection = new Connection(rpcClient);

            try
            {
                await _Authenticate(connectionData).ConfigureAwait(false);
            }
            catch(System.Exception)
            {
                await Disconnect().ConfigureAwait(false);
                throw;
            }
        }

        public Task Disconnect()
        {
            if (IsConnected)
            {
                _APIConnection.RpcClient?.Dispose();
            }

            _APIConnection = null;
            ConnectionData = null;
            ConnectionInfo = null;

            return Task.CompletedTask;
        }

        public Task Reconnect()
        {
            throw new NotImplementedException();
        }

        public async Task<ConnectionInfo> TestConnection(ConnectionData connectionData)
        {
            try
            {
                TcpRpcClient rpcClient = await _ConnectAsync(connectionData).ConfigureAwait(false);
                Connection testConnection = new Connection(rpcClient);

                rpcClient.Dispose();

                ConnectionInfo connectionInfo = new ConnectionInfo()
                {
                    APIVersion = testConnection.
                }

                return true;
            }
            catch
            {
                return null;
            }
        }
        #endregion

        #region Private Methods
        private static bool RemoteCertificateValidationCallback(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            // TODO Cert Check
            return true;
        }

        /// <summary>
        /// Connect to server async with ConnectionData
        /// </summary>
        /// <exception cref="AuthenticationException">TLS Error</exception>
        /// <exception cref="ConnectingFailedException">Based on RPC Exception</exception>
        /// 
        private async Task<TcpRpcClient> _ConnectAsync(ConnectionData connectionData)
        {
            TcpRpcClient rpcClient = new TcpRpcClient();
            rpcClient.InjectMidlayer((tcpstream) =>
            {
                var sslStream = new SslStream(tcpstream, false, new RemoteCertificateValidationCallback(RemoteCertificateValidationCallback));
                try
                {
                    sslStream.AuthenticateAsClient("bffhd");
                    return sslStream;
                }
                catch (AuthenticationException)
                {
                    sslStream.Close();
                    throw;
                }
            });
            
            try
            {
                rpcClient.Connect(connectionData.Host.Host, connectionData.Host.Port); 
                await rpcClient.WhenConnected.ConfigureAwait(false);

                return rpcClient;
            }
            catch (RpcException exception) when (string.Equals(exception.Message, "TcpRpcClient is unable to connect", StringComparison.Ordinal))
            {
                throw new ConnectingFailedException("Connecting failed", exception);
            }
        }

        /// <summary>
        /// Authenticate connection with ConnectionData
        /// </summary>
        /// <exception cref="UnsupportedMechanismException"></exception>
        /// <exception cref="InvalidCredentialsException"></exception>
        /// <exception cref="AuthenticationFailedException"></exception>
        private async Task _Authenticate(ConnectionData connectionData)
        {
            Dictionary<string, object> joinedProperties = new Dictionary<string, object>();
            foreach(KeyValuePair<string, object> keyValuePair in connectionData.Properties)
            {
                joinedProperties.Add(keyValuePair.Key, keyValuePair.Value);
            }
            foreach (KeyValuePair<string, object> keyValuePair in connectionData.SecretProperties)
            {
                joinedProperties.Add(keyValuePair.Key, keyValuePair.Value);
            }

            await _APIConnection.Auth(MechanismString.ToString(connectionData.Mechanism), joinedProperties).ConfigureAwait(false);
        }
        #endregion
    }
}
