﻿
using FabAccessAPI;
using FabAccessAPI.Exceptions;
using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace FabAccessAPI_Test
{
    public class API_Test
    {
        [TestCase("Admin1")]
        public async Task ConnectDisconnect(string username)
        {
            API api = new API();

            ConnectionData connectionData = new ConnectionData()
            {
                Host = new UriBuilder(TestEnv.SCHEMA, TestEnv.TESTSERVER, TestEnv.TESTSERVER_PORT).Uri,
                Mechanism = Mechanism.PLAIN,
                Username = username,
                Properties = new Dictionary<string, object>()
                {
                    { "Username", username }
                },
                SecretProperties = new Dictionary<string, object>()
                {
                    { "Password", TestEnv.PASSWORD }
                }
            };

            await api.Connect(connectionData);
            await api.Disconnect();
        }

        [Test]
        public async Task Connect_HostUnreachable()
        {
            API api = new API();

            ConnectionData connectionData = new ConnectionData()
            {
                Host = new UriBuilder(TestEnv.SCHEMA, "NotReachable." + TestEnv.TESTSERVER, TestEnv.TESTSERVER_PORT).Uri,
                Mechanism = Mechanism.PLAIN,
                Username = "UnknownUser",
                Properties = new Dictionary<string, object>()
                {
                    { "Username", "UnknownUser" }
                },
                SecretProperties = new Dictionary<string, object>()
                {
                    { "Password", TestEnv.PASSWORD }
                }
            };

            try
            {
                await api.Connect(connectionData);
            }
            catch (ConnectingFailedException)
            {
                Assert.Pass();
            }
            Assert.Fail();
        }

        [Test]
        public async Task Connect_InvalidCredentials()
        {
            API api = new API();

            ConnectionData connectionData = new ConnectionData()
            {
                Host = new UriBuilder(TestEnv.SCHEMA, TestEnv.TESTSERVER, TestEnv.TESTSERVER_PORT).Uri,
                Mechanism = Mechanism.PLAIN,
                Username = "UnknownUser",
                Properties = new Dictionary<string, object>()
                {
                    { "Username", "UnknownUser" }
                },
                SecretProperties = new Dictionary<string, object>()
                {
                    { "Password", TestEnv.PASSWORD }
                }
            };

            try
            {
                await api.Connect(connectionData);
            }
            catch(InvalidCredentialsException)
            {
                Assert.Pass();
            }
            Assert.Fail();
        }
    }
}
